<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

// This is a IA (chatgpt) powered conversational Bot. You must supply your 
// OpenAI API key for it to work. You will also need to modify the SYSTEM_PROMPT
// with data about your company/service/product. This will use the token limit 
// of ChatGPT, so you cannot have a very large prompt. Usually it is limited to
// 4096 tokens counting both the supplied prompt, the history context and the answer.
//
// For larger context we offer a paid bot option that uses RAG to expand knowledge
// almost without limits. Contact us for more info.

require_once("../chatbroker.class.".PHP_MAJOR_VERSION.".php");

$DEBUG        = 0;
$OPENAI_TOKEN = '';
$MODEL        = "gpt-3.5-turbo";

$GUARDRAILS_USER_PROMPT='. Do not justify your answer.';

$SYSTEM_PROMPT = "You are a helpful virtual assistant.";

$ACTION_PROMPT = "If the user is frustrated or angry, or if the user asks to connect to a real person, you will pass the conversation to a human by appending '[transfer]'. In the farewell/goodbye message you will append '[close]'.";

$TRANSCRIBE = "no";
$VOICEREPLY = "no";

// override all hardcoded options if we have a bot name and ini file
if(isset($_REQUEST['bot'])) {
    $filename = getcwd()."/".$_REQUEST['bot'].".ini";
    if(is_readable($filename)) {
        $ini = parse_ini_file($filename,false);
        $OPENAI_TOKEN = $ini['openai_token'];
        $SYSTEM_PROMPT = $ini['system_prompt'];
        $ACTION_PROMPT = $ini['action_prompt'];
        $GUARDRAILS_USER_PROMPT = $ini['guardrails_prompt'];
        $MODEL = $ini['model'];
        $TRANSCRIBE = $ini['transcribe_audio'];
        $VOICEREPLY = $ini['voice_reply'];
    }
}

$json = file_get_contents('php://input');
$data = json_decode($json,1);

log_debug("POST RECEIVED:\n".print_r($data,1));

// process audio attachments and transcribe using openai
if(isset($data['attach']['mime']) && $TRANSCRIBE=='yes') {
    $audio_file = preg_replace("/; codecs=opus/","",$data['attach']['file']);
    $mime = preg_replace("/; codecs=opus/","",$data['attach']['mime']);
    if(preg_match("/audio/",$mime)) {
        $cb = new ChatBroker(array());
        $files = $cb->check_for_attachments($data['msgid']);
        foreach($files as $file) {
            // retrieve file to temp disk to get metadata (mime type, image info, etc)
            $path = $cb->chatbroker_host ."/api/files/messages/".$data['msgid']."/".$file;
            $type = pathinfo($path, PATHINFO_EXTENSION);
            $filedata = file_get_contents($path);
            file_put_contents("/tmp/$file",$filedata);
            $text  = openai_whisper("/tmp/$file",$OPENAI_TOKEN);
            if($text!='') $data['body']=$text;
        }
    }
}

$user_prompt = $data['body'];

$user_prompt.=$GUARDRAILS_USER_PROMPT; // add guardrails to question

$chat_context = array_slice($data['chat_context'],-5);
$chat_context[] = array("role"=>"user","content"=>$user_prompt);

if($OPENAI_TOKEN=='') {
    $response = "Bot is not yet configured, be sure to set up your OpenAI API key";
} else {
    $response = ask_openai($OPENAI_TOKEN,$SYSTEM_PROMPT." ".$ACTION_PROMPT,$chat_context);
}

if($VOICEREPLY=='no') {
   $return = array("body"=>$response,"attach"=>'');
} else {
  $attach = openai_tts($response,"alloy",$OPENAI_TOKEN);
  if(is_null($attach)) {
    $return = array("body"=>$response,"attach"=>'');
  } else {
    $return = array("body"=>'',"attach"=>$attach);
  }
}

log_debug("REPLY: $response");
echo json_encode($return);
die();

function ask_openai($openai_token,$system_prompt,$user_prompt) {
    global $MODEL;

    $url = 'https://api.openai.com/v1/chat/completions';
    $sysprompt = array();
    $sysprompt[]=array("role"=>"system","content"=>$system_prompt);
    $messages = array_merge($sysprompt,$user_prompt);
    $data = array(
       "model"=> $MODEL,
       "messages" => $messages
    );
    $data_string = json_encode($data);
    log_debug("OpenAI Messages: ".print_r($messages,1));
    $headers=array('Content-Type: application/json','Content-Length: ' . strlen($data_string),"Authorization: Bearer $openai_token");
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_TIMEOUT, 120);
    curl_setopt($ch, CURLOPT_VERBOSE, true);
    curl_setopt($ch, CURLOPT_SSLVERSION, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $return = curl_exec($ch);
    curl_close($ch);
    $response = json_decode($return,1);
    log_debug("OpenAI response: ".print_r($response,1));
    return $response['choices'][0]['message']['content'];
}

function log_debug($text) {
    global $DEBUG;
    $logfile = basename(__FILE__,'.php');
    if($DEBUG==1) {
        file_put_contents("/tmp/${logfile}.log",$text."\n",FILE_APPEND);
    }
}

function openai_whisper($file,$token) {
  // file is the audio file to pass to whisper for transcription
  if(is_readable($file)) {
      $mime = mime_content_type($file);
      if (class_exists('CURLFile')) {
          $attachment = new CURLFile($file, $mime, basename($file));
      } else {
          $attachment = '@' . $file . ';filename=' . $file . ';type=' . $mime;
      }
      $headers=array('Authorization: Bearer '.$token, 'Content-Type: multipart/form-data');
      $ch = curl_init('https://api.openai.com/v1/audio/transcriptions');
      curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
      curl_setopt($ch, CURLOPT_HEADER, 0);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
      curl_setopt($ch, CURLOPT_TIMEOUT, 500);
      curl_setopt($ch, CURLOPT_VERBOSE, false);
      curl_setopt($ch, CURLOPT_SSLVERSION, 1);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($ch, CURLOPT_POST, 1);
      curl_setopt($ch, CURLOPT_POSTFIELDS, array(
          'model' => 'whisper-1',
          'response_format' => 'text',
          'file' => $attachment
      ));
      $transcript = curl_exec($ch);
      curl_close($ch);
      $transcript = trim($transcript);
      return $transcript;
  } else {
      return "";
  }
}

function openai_tts($text,$voice,$token) {

log_debug("tts $text");
    $text = removeEmojis($text);
log_debug("tts2 $text");
    $url = 'https://api.openai.com/v1/audio/speech';
    $data = [
        'model' => 'tts-1',
        'input' => $text,
        'voice' => $voice,
        'response_format' => 'opus',
        'prompt' => "Esta es una respuesta en español"
    ];
    $headers = [
        'Authorization: Bearer ' . $token,
        'Content-Type: application/json',
    ];

    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);

    $response = curl_exec($ch);
    $contentType = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
    log_debug("content type $contentType");

    if($contentType=="") {
        return null;
    }

    $tempFile = tempnam(sys_get_temp_dir(), 'audio_');
    $tempFile .= ".ogg";

    if (curl_errno($ch)) {
        echo 'Error:' . curl_error($ch);
    } else {
        file_put_contents($tempFile, $response);
    }

    curl_close($ch);

    return array(
        'file' => $tempFile,
        'mime' => "audio/ogg",
        'filename' => "speech.ogg"
    );
}

function removeEmojis($text) {
    // Input validation
    if (!is_string($text)) {
        throw new InvalidArgumentException('Input must be a string');
    }

    // Return empty string if input is empty
    if (empty($text)) {
        return '';
    }

    // Pattern to match emoji characters
    $pattern = '/[\x{1F000}-\x{1F9FF}]'  // Miscellaneous Symbols and Pictographs
             . '|[\x{2600}-\x{26FF}]'    // Miscellaneous Symbols
             . '|[\x{2700}-\x{27BF}]'    // Dingbats
             . '|[\x{2300}-\x{23FF}]'    // Miscellaneous Technical
             . '|[\x{2B50}]'             // White Medium Star
             . '|[\x{2934}-\x{2935}]'    // Arrow characters
             . '|[\x{2B05}-\x{2B07}]'    // Directional arrows
             . '|[\x{2B1B}-\x{2B1C}]'    // Black and white squares
             . '|[\x{3030}]'             // Wavy dash
             . '|[\x{303D}]'             // Part alternation mark
             . '|[\x{3297}]'             // Circled ideograph congratulation
             . '|[\x{3299}]'             // Circled ideograph secret
             . '|[\x{FE0F}]'             // Variation selector
             . '|[\x{1F600}-\x{1F64F}]'  // Emoticons
             . '|[\x{1F300}-\x{1F5FF}]'  // Misc Symbols and Pictographs
             . '|[\x{1F680}-\x{1F6FF}]'  // Transport and Map
             . '|[\x{1F900}-\x{1F9FF}]'  // Supplemental Symbols and Pictographs
             . '|[\x{1F1E0}-\x{1F1FF}]'  // Flags (iOS)
             . '/u';                      // UTF-8 flag

    // Remove emojis using preg_replace
    $cleanText = preg_replace($pattern, '', $text);
    
    // Remove any zero-width joiner characters that might be left
    $cleanText = preg_replace('/[\x{200D}]/u', '', $cleanText);
    
    return $cleanText;
}


